import spatiafi.gdal_auth
from spatiafi.auth import authenticate
from spatiafi.session import get_session

try:
    # -- Distribution mode --
    # import from _version.py generated by setuptools_scm during release
    from ._version import version as __version__
except ImportError:
    # -- Source mode --
    # use setuptools_scm to get the current version from src using git
    from setuptools_scm import get_version as _gv

    __version__ = _gv(relative_to=__file__, search_parent_directories=True)



import os

from authlib.integrations.httpx_client import AsyncOAuth2Client
from authlib.integrations.requests_client import OAuth2Session
from authlib.oauth2.rfc7523 import ClientSecretJWT

from spatiafi import authenticate


def load_or_authenticate():
    """Load credentials from environment variables or authenticate to get them."""
    if os.environ.get("SPATIAFI_CLIENT_ID") and os.environ.get(
        "SPATIAFI_CLIENT_SECRET"
    ):
        return {
            "client_id": os.environ["SPATIAFI_CLIENT_ID"],
            "client_secret": os.environ["SPATIAFI_CLIENT_SECRET"],
        }
    else:
        return authenticate()


def get_session(app_credentials=None):
    """Get an automatically-refreshing OAuth2 session (requests, sync) for the SpatiaFI API."""

    if app_credentials is None:
        app_credentials = load_or_authenticate()

    client_id = app_credentials["client_id"]
    client_secret = app_credentials["client_secret"]

    session = OAuth2Session(
        client_id,
        client_secret,
        token_endpoint="https://auth.spatiafi.com/api/v1/auth/jwt/token",
        grant_type="client_credentials",
        token_endpoint_auth_method=ClientSecretJWT(
            "https://auth.spatiafi.com/api/v1/auth/jwt/token"
        ),
    )
    session.fetch_token()
    return session


async def get_async_session(app_credentials=None):
    """Get an automatically-refreshing async OAuth2 session for the SpatiaFI API."""

    if app_credentials is None:
        app_credentials = load_or_authenticate()

    client_id = app_credentials["client_id"]
    client_secret = app_credentials["client_secret"]

    session = AsyncOAuth2Client(
        client_id,
        client_secret,
        token_endpoint="https://auth.spatiafi.com/api/v1/auth/jwt/token",
        grant_type="client_credentials",
        token_endpoint_auth_method=ClientSecretJWT(
            "https://auth.spatiafi.com/api/v1/auth/jwt/token"
        ),
    )
    await session.fetch_token()
    return session


if __name__ == "__main__":
    get_session()



"""
Setting up GDAL to authenticate to GCS can be a pain if you don't use a downloaded service account key file,
which is a pretty bad security practice. This module sets up GDAL to authenticate to GCS using the
Google Application Default Credentials (ADC) strategy. This means that if you're running on GCP, GDAL will
authenticate using the instance's service account. If you're running locally, GDAL will authenticate using
the credentials in your local user's Application Default Credentials (ADC) file (usually this is user
credentials from `gcloud auth application-default login`).
"""
import json
import logging
import os
import webbrowser
from typing import Dict, Tuple

import google.auth
import google.auth.credentials
import google.auth.transport.requests
import requests
from google.oauth2.credentials import Credentials as UserCredentials
from platformdirs import user_config_dir

logger = logging.getLogger(__name__)

_OAUTH_CONFIG = {
    "web": {
        "client_id": "111941376227-3kjeanolo61g8t207od52epjinga0gdo.apps.googleusercontent.com",
        "project_id": "ce-datasets",
        "auth_uri": "https://accounts.google.com/o/oauth2/auth",
        "token_uri": "https://oauth2.googleapis.com/token",
        "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
        "client_secret": "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",
        "redirect_uris": ["https://auth.spatiafi.com/google_auth_helper/"],
    }
}


_GDAL_SCOPES = [
    "https://www.googleapis.com/auth/devstorage.read_write",
    "https://www.googleapis.com/auth/userinfo.email",
    "https://www.googleapis.com/auth/userinfo.profile",
    "https://www.googleapis.com/auth/accounts.reauth",
    "https://www.googleapis.com/auth/earthengine",
    "openid",
]

_on_gcp = None

on_gcp_file = os.path.join(
    user_config_dir("spatiafi", ensure_exists=True), "on_gcp.txt"
)

user_credentials_file = os.path.join(
    user_config_dir("spatiafi", ensure_exists=True), "gcp_user_credentials.json"
)


def on_gcp():
    global _on_gcp
    if _on_gcp is not None:
        return _on_gcp

    if os.path.exists(on_gcp_file):
        with open(on_gcp_file, "r") as f:
            _on_gcp = f.read().strip() == "True"
        return _on_gcp

    logger.debug("Checking if on GCP")
    metadata_url = "http://metadata.google.internal/computeMetadata/v1/"
    metadata_headers = {"Metadata-Flavor": "Google"}

    try:
        response = requests.head(metadata_url, headers=metadata_headers, timeout=1)

        if response.status_code == requests.codes.ok:
            with open(on_gcp_file, "w") as f:
                f.write("True")
            _on_gcp = True
            return True

    except requests.exceptions.RequestException:
        # Ignore the exception, connection failed means the attribute does not
        # exist in the metadata server.
        pass

    with open(on_gcp_file, "w") as f:
        f.write("False")
    _on_gcp = False
    return False


def oob_auth_flow(open_browser=True, project=None):
    from google_auth_oauthlib.flow import Flow

    # Create the flow using the client secrets file from the Google API console.
    flow = Flow.from_client_config(
        _OAUTH_CONFIG,
        scopes=_GDAL_SCOPES,
    )
    flow.redirect_uri = _OAUTH_CONFIG["web"]["redirect_uris"][0]

    auth_url, _ = flow.authorization_url(prompt="consent")
    if open_browser:
        webbrowser.open(auth_url, new=1, autoraise=True)

    print("Please visit this URL to authorize this application: {}".format(auth_url))
    code = input("Enter the authorization code: ")

    flow.fetch_token(code=code)

    credentials = flow.credentials

    return credentials, flow.client_config["project_id"]


def get_user_credentials(
    project=None,
) -> Tuple[google.auth.credentials.Credentials, str]:
    """Get or refresh credentials"""

    if os.path.exists(user_credentials_file):
        with open(user_credentials_file, "r") as f:
            info = json.load(f)

        quota_project_id = info.get("quota_project_id", None)
        credentials = UserCredentials.from_authorized_user_info(info=info)

        if project and project != quota_project_id:
            credentials = credentials.with_quota_project(quota_project_id=project)
        if credentials.expired:
            credentials.refresh(google.auth.transport.requests.Request())
        return credentials, project

    credentials, project = oob_auth_flow(project=project)

    credentials.refresh(
        google.auth.transport.requests.Request()
    )  # need to call once to get token

    if isinstance(credentials, UserCredentials):
        with open(user_credentials_file, "w") as f:
            credentials_dict = json.loads(credentials.to_json())
            if project:
                credentials_dict["quota_project_id"] = project
            f.write(json.dumps(credentials_dict))

    return credentials, project


def get_user_auth_env_vars(project=None):
    """Returns a dictionary of environment variables needed for GDAL to auth to GCS"""
    credentials, project = get_user_credentials(project=project)

    return {
        "EEDA_BEARER": credentials.token,
        "GS_OAUTH2_REFRESH_TOKEN": credentials.refresh_token,
        "GS_OAUTH2_CLIENT_ID": credentials.client_id,
        "GS_OAUTH2_CLIENT_SECRET": credentials.client_secret,
    }


def get_gdal_env_vars(project=None) -> Dict[str, str]:
    if on_gcp():
        logger.debug("Running on GCP, using instance service account")
        env = {"CPL_MACHINE_IS_GCE": "YES"}
    else:
        logger.debug("Running outside GCP, using ADC")
        env = get_user_auth_env_vars(project=project)

    return env


def get_credentials(project=None) -> Tuple[google.auth.credentials.Credentials, str]:
    if on_gcp():
        return google.auth.default(quota_project_id=project)
    else:
        return get_user_credentials(project=project)


if __name__ == "__main__":
    import pprint

    pprint.pprint(get_gdal_env_vars())



import json
import os

import requests
from platformdirs import user_config_dir

app_credentials_file = os.path.join(
    user_config_dir("spatiafi", ensure_exists=True), "app_credentials.json"
)


def get_basic_auth_token(username, password):
    """Get an access token using HTTP Basic Authentication."""
    url = "https://auth.spatiafi.com/api/v1/auth/jwt/token/basic"
    response = requests.get(url, auth=(username, password))
    response_data = response.json()
    return response_data["access_token"]


def prompt_for_basic_auth_token():
    """Prompt for username and password and get an access token using HTTP Basic Authentication."""
    username = input("Username: ")
    password = input("Password: ")
    token = get_basic_auth_token(username, password)
    print("Got temporary access token")
    return token


def get_new_app_credentials(access_token=None):
    """Get new app credentials from the SpatiaFI Auth API."""

    if access_token is None:
        access_token = prompt_for_basic_auth_token()

    headers = {"Authorization": "Bearer " + access_token}

    response = requests.post(
        "https://auth.spatiafi.com/api/v1/clients/", headers=headers
    )
    response_data = response.json()
    store_app_credentials(response_data)
    print(f"Got new App Credentials and saved them to: {app_credentials_file}")
    return response_data


def store_app_credentials(app_credentials):
    """Store app credentials to a file."""
    with open(app_credentials_file, "w") as f:
        json.dump(app_credentials, f)


def load_app_credentials():
    """Load app credentials from a file."""
    with open(app_credentials_file, "r") as f:
        app_credentials = json.load(f)
    return app_credentials


def load_app_credentials_into_env(app_credentials):
    """Load SpatiaFI app credentials into environment variables."""
    os.environ["SPATIAFI_CLIENT_ID"] = app_credentials["client_id"]
    os.environ["SPATIAFI_CLIENT_SECRET"] = app_credentials["client_secret"]


def authenticate():
    """Try to load app credentials from a file, otherwise get new app credentials."""
    try:
        app_credentials = load_app_credentials()
    except FileNotFoundError:
        app_credentials = get_new_app_credentials()
    load_app_credentials_into_env(app_credentials)
    return app_credentials


if __name__ == "__main__":
    authenticate()



import sys
import tempfile
from pathlib import Path
from typing import Dict

import click

from spatiafi.gdal_auth.gdal_auth import get_gdal_env_vars

GDAL_CLI_TOOLS = [
    "gdalinfo",
    "gdal_translate",
    "gdalwarp",
    "gdalbuildvrt",
    "gdaladdo",
    "gdal_grid",
    "gdallocationinfo",
    "gdalmanage",
    "gdalsrsinfo",
    "gdaltindex",
    "gdaltransform",
    "gdal_contour",
    "gdal_rasterize",
    "gdal_edit",
    "gdal_merge",
    "gdal_pansharpen",
    "gdal_polygonize",
    "gdal_proximity",
    "gdal_retile",
    "gdal_sieve",
    "gdalcompare",
    "gdalident",
    "gdallocationinfo",
    "gdalmanage",
]


def set_env_source_file(env: Dict[str, str]):
    """Sets environment variables using a source file"""
    # create a temporary file to store the env vars
    env_file = Path(tempfile.gettempdir()) / "gdal_auth.env"

    with env_file.open("w") as f:
        for k, v in env.items():
            f.write(f"export {k}={v}\n")

    # Instructions for setting the env vars
    print("Env vars saved to: ", env_file)
    print("Run the following command in your command line to set the env vars:\n")
    print(f"source {env_file}")
    print("")


def set_env_one_line_command(env: Dict[str, str], pre=False):
    """Sets environment variables using a one line command"""
    if not pre:
        print("Run the following command in your command line to set the env vars:\n")
    print("export " + " ".join([f"{k}={v}" for k, v in env.items()]), ";")


def print_alias_instructions(project=None):
    """Prints instructions for setting up an alias"""
    # set alias file to ~/.gdal_aliases
    alias_file = Path.home() / ".gdal_aliases"
    this_file = Path(__file__).absolute()
    py_exe = Path(sys.executable).absolute() if sys.executable else "python"
    project_arg = f"--project {project}" if project else ""
    bash_func = (
        f'gdal_auth_hook() {{ eval "$({py_exe} {this_file} --pre {project_arg})"; }}'
    )

    with open(alias_file, "w") as f:
        f.write(bash_func + "\n")
        f.write("export -f gdal_auth\n")
        for gdal_cmd in GDAL_CLI_TOOLS:
            alias_cmd = f'alias {gdal_cmd}="gdal_auth_hook && {gdal_cmd}"'
            f.write(alias_cmd + "\n")

    print("Run the following command in your command line to set the aliases:\n")
    print(f"source {alias_file}")
    print("")
    print(
        "After the aliases are set, you can use GDAL commands as normal, e.g. `gdalinfo gs://my-bucket/my-file.tif`"
    )
    print("and authentication will be handled automatically")
    print("")
    print(
        f"You can also add 'source {alias_file}' line to your ~/.bashrc file to load the aliases on startup"
    )


@click.command()
@click.option("-f", "--file", is_flag=True, help="Create an env file")
@click.option("-l", "--line", is_flag=True, help="Print a one line command")
@click.option("-a", "--alias", is_flag=True, help="Print alias instructions")
@click.option(
    "-p",
    "--project",
    default=None,
    help="GCP project ID to use for billing (default: from ADC)",
)
@click.option(
    "--pre", is_flag=True, help="Run as a pre-command to a GDAL/rio (used by aliases)"
)
def cli(file, line, alias, project, pre):
    """Set environment variables needed for GDAL to authenticate to Google Cloud"""
    env = get_gdal_env_vars(project=project)
    if file:
        set_env_source_file(env)
    elif line:
        set_env_one_line_command(env)
    elif alias:
        print_alias_instructions(project=project)
    elif pre:
        set_env_one_line_command(env, pre=True)
    else:
        print("Set the following environment variables:\n")
        for k, v in env.items():
            print(f"{k}={v}")


if __name__ == "__main__":
    cli()
